Hooks:PostHook(ConcussionGrenade, "_setup_from_tweak_data", "add_new_selves_for_beeps", function(self)
    self._state = 0

    local light = World:create_light("omni|specular")

	light:set_far_range(300)
	light:set_color(Color(1, 0, 0))
	light:set_position(self._unit:position())
	light:set_specular_multiplier(1)
	light:set_enable(true)
	light:set_multiplier(0)
	light:set_falloff_exponent(0.5)

	self._light = light
	self._light_multiplier = 0

	self._sound_source = SoundDevice:create_source("concussion_beep_source")
end)

function ConcussionGrenade:update(unit, t, dt)

    if self._timer then
		self._timer = self._timer - dt

        if self._timer <= 0 then
            self._state = self._state + 1
			self._timer = nil

			self:_detonate()

			return
        end
	end
	
	if self._beep_t then
		self._beep_t = self._beep_t - dt

		if self._beep_t < 0 then
			self:_beep()
		end
	else
		self:_beep()
	end
    
    if alive(self._light) then
		self._light_multiplier = math.clamp(self._light_multiplier - dt * 4, 0, 1)
		self._light:set_multiplier(self._light_multiplier)
		self._light:set_far_range(500 * self._light_multiplier)
	end

	GrenadeBase.super.update(self, unit, t, dt)
end

function ConcussionGrenade:timer()
    return self._timer
end

function ConcussionGrenade:_get_next_beep_time()
	local beep_speed = {
        0.1,
        0.025
    }

	return self:timer() / beep_speed[1] * beep_speed[2]
end

function ConcussionGrenade:_beep()
	self._sound_source:set_position(self._unit:position())
	self._light:set_position(self._unit:position())

	self._sound_source:post_event("pfn_beep")

	self._beep_t = self:_get_next_beep_time()
	self._light_multiplier = 0.6
end

Hooks:PostHook(ConcussionGrenade, "_detonate", "post_detonate_cleanup", function(self)
	if alive(self._light) then
		World:delete_light(self._light)

		self._light = nil
	end
end)